﻿using Microsoft.EntityFrameworkCore;
using DinerContracts.BindingModels;
using DinerContracts.SearchModels;
using DinerContracts.StoragesContracts;
using DinerContracts.ViewModels;
using DinerDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DinerDatabaseImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        public List<ShopViewModel> GetFullList()
        {
            using var context = new DinerDatabase();
            return context.Shops.Include(x => x.Snacks).ThenInclude(x => x.Snack).ToList().
                Select(x => x.GetViewModel).ToList();
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            using var context = new DinerDatabase();
            return context.Shops.Include(x => x.Snacks).ThenInclude(x => x.Snack).Where(x => x.ShopName.Contains(model.ShopName)).
                ToList().Select(x => x.GetViewModel).ToList();
        }

        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return new();
            }
            using var context = new DinerDatabase();
            return context.Shops.Include(x => x.Snacks).ThenInclude(x => x.Snack)
                .FirstOrDefault(x =>
                (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) ||
                (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            using var context = new DinerDatabase();
            var newShop = Shop.Create(context, model);
            if (newShop == null)
            {
                return null;
            }
            context.Shops.Add(newShop);
            context.SaveChanges();
            return newShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            using var context = new DinerDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var shop = context.Shops.FirstOrDefault(x => x.Id == model.Id);
                if (shop == null)
                {
                    return null;
                }
                shop.Update(model);
                context.SaveChanges();
                shop.UpdateSnacks(context, model);
                transaction.Commit();
                return shop.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public ShopViewModel? Delete(ShopBindingModel model)
        {
            using var context = new DinerDatabase();
            var shop = context.Shops.Include(x => x.Snacks).FirstOrDefault(x => x.Id == model.Id);
            if (shop != null)
            {
                context.Shops.Remove(shop);
                context.SaveChanges();
                return shop.GetViewModel;
            }
            return null;
        }

        public bool RestockingShops(SupplyBindingModel model)
        {
            using var context = new DinerDatabase();
            var transaction = context.Database.BeginTransaction();
            var Shops = context.Shops.Include(x => x.Snacks).ThenInclude(x => x.Snack).ToList().
                Where(x => x.SnackMaxCount > x.ShopSnacks.Select(x => x.Value.Item2).Sum()).ToList();
            if (model == null)
            {
                return false;
            }
            try
            {
                foreach (Shop shop in Shops)
                {
                    int difference = shop.SnackMaxCount - shop.ShopSnacks.Select(x => x.Value.Item2).Sum();
                    int refill = Math.Min(difference, model.Count);
                    model.Count -= refill;
                    if (shop.ShopSnacks.ContainsKey(model.SnackId))
                    {
                        var datePair = shop.ShopSnacks[model.SnackId];
                        datePair.Item2 += refill;
                        shop.ShopSnacks[model.SnackId] = datePair;
                    }
                    else
                    {
                        var snack = context.Snacks.First(x => x.Id == model.SnackId);
                        shop.ShopSnacks.Add(model.SnackId, (snack, refill));
                    }
                    shop.SnacksDictionatyUpdate(context);
                    if (model.Count == 0)
                    {
                        transaction.Commit();
                        return true;
                    }
                }
                transaction.Rollback();
                return false;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public bool Sale(SupplySearchModel model)
        {
            using var context = new DinerDatabase();
            var transaction = context.Database.BeginTransaction();
            try
            {
                var shops = context.Shops.Include(x => x.Snacks).ThenInclude(x => x.Snack).ToList().
                    Where(x => x.ShopSnacks.ContainsKey(model.SnackId.Value)).OrderByDescending(x => x.ShopSnacks[model.SnackId.Value].Item2).ToList();

                foreach (var shop in shops)
                {
                    int residue = model.Count.Value - shop.ShopSnacks[model.SnackId.Value].Item2;
                    if (residue > 0)
                    {
                        shop.ShopSnacks.Remove(model.SnackId.Value);
                        shop.SnacksDictionatyUpdate(context);
                        context.SaveChanges();
                        model.Count = residue;

                    }
                    else
                    {
                        if (residue == 0)
                            shop.ShopSnacks.Remove(model.SnackId.Value);
                        else
                        {
                            var dataPair = shop.ShopSnacks[model.SnackId.Value];
                            dataPair.Item2 = -residue;
                            shop.ShopSnacks[model.SnackId.Value] = dataPair;
                        }

                        shop.SnacksDictionatyUpdate(context);
                        transaction.Commit();
                        return true;
                    }
                }
                transaction.Rollback();
                return false;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }


    }
}
